@@ -1,14 +1,15 @@
 package CPAN::Checksums;
 
 use strict;
-use vars qw($VERSION $CAUTION @ISA @EXPORT_OK);
+use vars qw($VERSION $CAUTION $TRY_SHORTNAME @ISA @EXPORT_OK);
 
 require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT_OK = qw(updatedir);
-$VERSION = sprintf "%d.%03d", q$Revision: 1.9 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%03d", q$Revision: 1.11 $ =~ /(\d+)\.(\d+)/;
 $CAUTION ||= 0;
+$TRY_SHORTNAME ||= 0;
 
 use DirHandle ();
 use IO::File ();
@@ -31,48 +32,57 @@ sub updatedir ($) {
     next if $de =~ /readme$/i;
 
     my $abs = File::Spec->catfile($dirname,$de);
-    next if -l $abs;
 
     #
     # SHORTNAME offers an 8.3 name, probably not needed but it was
     # always there,,,
     #
-    my $shortname = lc $de;
-    $shortname =~ s/\.tar[._-]gz$/\.tgz/;
-    my $suffix;
-    ($suffix = $shortname) =~ s/.*\.//;
-    substr($suffix,3) = "" if length($suffix) > 3;
-    if ($shortname =~ /\-/) {
-      @p = $shortname =~ /(.{1,16})-.*?([\d\.]{2,8})/;
-    } else {
-      @p = $shortname =~ /(.{1,8}).*?([\d\.]{2,8})/;
-    }
-    $p[0] ||= lc $de;
-    $p[0] =~ s/[^a-z0-9]//g;
-    $p[1] ||= 0;
-    $p[1] =~ s/\D//g;
-    my $counter = 7;
-    while (length($p[0]) + length($p[1]) > 8) {
-      substr($p[0], $counter) = "" if length($p[0]) > $counter;
-      substr($p[1], $counter) = "" if length($p[1]) > $counter--;
-    }
-    my $dot = $suffix ? "." : "";
-    $shortname = "$p[0]$p[1]$dot$suffix";
-    while (exists $shortnameseen{$shortname}) {
-      my($modi) = $shortname =~ /([a-z\d]+)/;
-      $modi++;
-      $shortname = "$modi$dot$suffix";
-      if ($counter++ > 1000){ # avoid endless loops and accept the buggy choice
-        warn "Warning: long loop on shortname[$shortname]de[$de]";
-        last;
+    if ($TRY_SHORTNAME) {
+      my $shortname = lc $de;
+      $shortname =~ s/\.tar[._-]gz$/\.tgz/;
+      my $suffix;
+      ($suffix = $shortname) =~ s/.*\.//;
+      substr($suffix,3) = "" if length($suffix) > 3;
+      if ($shortname =~ /\-/) {
+        @p = $shortname =~ /(.{1,16})-.*?([\d\.]{2,8})/;
+      } else {
+        @p = $shortname =~ /(.{1,8}).*?([\d\.]{2,8})/;
       }
+      $p[0] ||= lc $de;
+      $p[0] =~ s/[^a-z0-9]//g;
+      $p[1] ||= 0;
+      $p[1] =~ s/\D//g;
+      my $counter = 7;
+      while (length($p[0]) + length($p[1]) > 8) {
+        substr($p[0], $counter) = "" if length($p[0]) > $counter;
+        substr($p[1], $counter) = "" if length($p[1]) > $counter--;
+      }
+      my $dot = $suffix ? "." : "";
+      $shortname = "$p[0]$p[1]$dot$suffix";
+      while (exists $shortnameseen{$shortname}) {
+        my($modi) = $shortname =~ /([a-z\d]+)/;
+        $modi++;
+        $shortname = "$modi$dot$suffix";
+        if (++$counter > 1000){ # avoid endless loops and accept the buggy choice
+          warn "Warning: long loop on shortname[$shortname]de[$de]";
+          last;
+        }
+      }
+      $dref->{$de}->{shortname} = $shortname;
+      $shortnameseen{$shortname} = undef; # for exists check good enough
     }
-    $dref->{$de}->{shortname} = $shortname;
-    $shortnameseen{$shortname} = undef; # for exists check good enough
 
     #
     # STAT facts
     #
+    if (-l File::Spec->catdir($dirname,$de)){
+      # Symlinks are a mess on a replicated, database driven system,
+      # but as they are not forbidden, we cannot ignore them. We do
+      # have a directory with nothing but a symlink in it. When we
+      # ignored the symlink, we did not write a CHEKSUMS file and
+      # CPAN.pm issued lots of warnings:-(
+      $dref->{$de}{issymlink} = 1;
+    }
     if (-d File::Spec->catdir($dirname,$de)){
       $dref->{$de}{isdir} = 1;
     } else {
@@ -182,6 +192,7 @@ sub makehashref ($) {
 }
 
 1;
+
 __END__
 
 =head1 NAME
@@ -193,17 +204,34 @@ CPAN::Checksums - Write a CHECKSUMS file for a directory as on CPAN
   use CPAN::Checksums qw(updatedir);
   my $success = updatedir($directory);
 
+=head1 INCOMPATIBILITY ALERT
+
+Since version 1.0 the generation of the attribute C<shortname> is
+turned off by default. It was too slow and was not used as far as I
+know, and above all, it could fail on large directories. The shortname
+feature can still be turned on by setting the global variable
+$TRY_SHORTNAME to a true value.
+
 =head1 DESCRIPTION
 
-updatedir takes a directory name as argument and writes a CHECKSUMS
-file in that directory unless a previously written CHECKSUMS file is
-there that is still valid. Returns 2 if a new CHECKSUMS file has been
-written, 1 if a valid CHECKSUMS file is already there, otherwise dies.
+updatedir takes a directory name as argument and writes a typical
+CHECKSUMS file in that directory as used on CPAN unless a previously
+written CHECKSUMS file is there that is still valid. Returns 2 if a
+new CHECKSUMS file has been written, 1 if a valid CHECKSUMS file is
+already there, otherwise dies. Please note, that CPAN CHECKSUMS files
+ignore filenames matching C</readme$/i>.
 
-Setting the global variable $CAUTION causes updatedir to report
+Setting the global variable $CAUTION causes updatedir() to report
 changes of files in the attributes C<size>, C<mtime>, C<md5>, or
 C<md5-ungz> to STDERR.
 
+By setting the global variable $TRY_SHORTNAME to a true value, you can
+tell updatedir() to include an attribute C<shortname> in the resulting
+hash that is 8.3-compatible. Please note, that updatedir() in this
+case may be slow and may even fail on large directories, because it
+will always only try 1000 iterations to find a name that is not yet
+taken and then give up.
+
 =head1 PREREQUISITES
 
 DirHandle, IO::File, Digest::MD5, Compress::Zlib, File::Spec,
